#!/usr/bin/env python
#   -*- encoding:   UTF-8   -*-

"""
    'The Web is Dead' (TWID) is a game for PyWeek 13
    Copyright (C) 2011 D Haynes

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    tundish@thuswise.org
"""

class Model(object):

    rTrans = 9E-9
    rRcvry = 4E-4
    rInfct=1E-2

    def __init__(self, rTrans=rTrans, rRcvry=rRcvry, popn=1E6):
        self.dT = 1.0
        self._t = 0
        self.tEnd = 20 * 60 * 60

        self.rTrans = rTrans
        self.rRcvry = rRcvry
        self.susbl = popn
        self.infct = 0
        self.rcvrd = 0
    
    @property
    def settings(self):
        return (self.rTrans, self.rRcvry)

    @property
    def state(self):
        return (self._t, self.susbl, self.infct, self.rcvrd)

    def start(self, rInfct=rInfct):
        if not self.infct:
            self.infct = int(rInfct * self.susbl)
            self.susbl -= self.infct

    def set(self, rTrans=rTrans, rRcvry=rRcvry):
        self.rTrans = rTrans
        self.rRcvry = rRcvry
        
    def step(self, delta=None):
        delta = delta or self.dT
        self._t += delta
        if self._t > self.tEnd or self.infct == 0:
            return 0

        newlyInfected = min(self.susbl, int(self.rTrans * self.susbl * self.infct))
        newlyRecovered = int(self.rRcvry * self.infct) or self.infct
        self.susbl, self.infct, self.rcvrd = (
            self.susbl - newlyInfected, 
            self.infct + newlyInfected - newlyRecovered, 
            self.rcvrd + newlyRecovered 
        )

        return delta

